#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "math/Complex.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480
#define MAX_ITERATIONS  256

void drawMandelbrot(int offsetx, int offsety, Color* palette)
{
    for (int y = 0; y < SCREEN_HEIGHT; y += 8)
    {
        for (int x = 0; x < SCREEN_WIDTH; x += 8)
        {
            // convert the pixel pos to a complex number
            Complex c;
            c.re = (x + offsetx - SCREEN_WIDTH  / 2.0) * 4.0 / SCREEN_WIDTH - 0.7;
            c.im = (y + offsety - SCREEN_HEIGHT / 2.0) * 4.0 / SCREEN_WIDTH;

            Complex z;
            int iteration = 0;

            while (z.squareMag() <= 16 &&
                   iteration < MAX_ITERATIONS)
            {
                // compute the new value
                z = z * z + c;

                iteration++;
            }

            // draw the pixel
            Color   col;

            if (iteration == MAX_ITERATIONS)
            {
                int v = (int)(z.mag() * 128.0) % 256;
                col = palette[v];
            }
            else
            {
                col = Color(0, 0, 0);
            }

            gfx.setPixel(x + offsetx, y + offsety, col);
        }
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Mandelbrot 5", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    static Uint8 pattern[64] =
    {
         0, 32,  8, 40,  2, 34, 10, 42,
        48, 16, 56, 24, 50, 18, 58, 26,
        12, 44,  4, 36, 14, 46,  6, 38,
        60, 28, 52, 20, 62, 30, 54, 22,
         3, 35, 11, 43,  1, 33,  9, 41,
        51, 19, 59, 27, 49, 17, 57, 25,
        15, 47,  7, 39, 13, 45,  5, 37,
        63, 31, 55, 23, 61, 29, 53, 21
    };

    static Color    palette[256];

    // set up the palette
    for (int i = 0; i < 256; ++i)
    {
        float angle = 2.0 * M_PI * (float)i / 256.0;
        float offset = 2.0 * M_PI / 3.0;
        angle += 40.0 *  M_PI / 128.0;
        angle *= 1.3;

        palette[i].r = 128 + 128 * sin(angle);
        palette[i].g = 128 + 128 * sin(angle * 1.1 - offset);
        palette[i].b = 128 + 128 * sin(angle * 1.5 + offset);
    }

    for (int i = 0; i < 64; i++)
    {
        int j = 0;
        while (pattern[j] != i)
            j++;

        drawMandelbrot(j % 8, j / 8, palette);

        // display and process the events
        gfx.render();
        sys.wait(50);
        sys.processEvents();
        if (sys.isQuitRequested() == true)
            exit(EXIT_SUCCESS);
    }

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        sys.processEvents();
        sys.wait(50);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
